#include <linux/list.h>
#include <linux/device.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/sysdev.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/mach/irq.h>

#include "BuffaloGpio.h"
#include "mvGpp.h"
#include "mvGppRegs.h"
#include "mvEthPhy.h"
#include "mvBoardEnvLib.h"
#include "mvUart.h"

//#define CONFIG_BUFFALO_GPIO_DRIVER_DEBUG

#if defined CONFIG_BUFFALO_GPIO_DRIVER_DEBUG
	#define BUF_TRACE(x)	x
#else
	#define BUF_TRACE(x)
#endif

void BuffaloGpio_EthLedOn(void);
void BuffaloGpio_EthLedOff(void);

#define ENTERED		BUF_TRACE(printk("%s > Entered\n", __FUNCTION__))
#define LEAVING(x)	BUF_TRACE(printk("%s > Leaving(ret=%d)\n", __FUNCTION, x))

unsigned int BuffaloGpio_CPUGetResetFlag(void){
#define CPU_RESET_REG	0x20104
#define CPU_RESET_BIT	BIT(1)

	unsigned int CpuResetFlag = MV_REG_READ(CPU_RESET_REG);
	return (CpuResetFlag & CPU_RESET_BIT);

}

void BuffaloGpio_CPUInterruptsSetup(void)
{
	mvGppOutEnablle(0, BIT(BIT_POWER), BIT(BIT_POWER));
	mvGppPolaritySet(0, BIT(BIT_POWER), (MV_GPP_IN_INVERT & BIT(BIT_POWER)));
}

void BuffaloGpio_CPUInterruptsClear(void)
{
	unsigned int cause;
	printk("MPP_CONTROL_REG 0x%04x 0x%04x 0x%04x 0x%04x\n",
		MV_REG_READ(mvMppRegGet(0)),
		MV_REG_READ(mvMppRegGet(1)),
		MV_REG_READ(mvMppRegGet(2)),
		MV_REG_READ(mvMppRegGet(3)));

	cause = MV_REG_READ(GPP_INT_CAUSE_REG(0));
	printk("cause:0x%04x\n", cause);
	MV_REG_WRITE(GPP_INT_CAUSE_REG(0), ~(cause & BIT(BIT_POWER)));
	BuffaloGpio_EthLedOn();
}

void BuffaloGpio_CPUInterruptsSetupInit(void)
{
	mvGppOutEnablle(0, BIT(BIT_INIT), BIT(BIT_INIT));
	mvGppPolaritySet(0, BIT(BIT_INIT), BIT(BIT_INIT));
}

void BuffaloGpio_CPUInterruptsClearInit(void)
{
	unsigned int cause;
	printk("MPP_CONTROL_REG 0x%04x 0x%04x 0x%04x 0x%04x\n",
		MV_REG_READ(mvMppRegGet(0)),
		MV_REG_READ(mvMppRegGet(1)),
		MV_REG_READ(mvMppRegGet(2)),
		MV_REG_READ(mvMppRegGet(3)));

	cause = MV_REG_READ(GPP_INT_CAUSE_REG(0));
	printk("cause:0x%04x\n", cause);
	MV_REG_WRITE(GPP_INT_CAUSE_REG(0), ~(cause & BIT(BIT_INIT)));
}

unsigned int buffalo_gpio_read(void)
{
	return (unsigned int) mvGppValueGet(0, -1);	
}

unsigned int buffalo_gpio_blink_reg_read(void){
	return (unsigned int) MV_REG_READ(GPP_BLINK_EN_REG(0));
}

void BuffaloGpio_Init(void){
	BUF_TRACE(printk("%s > Present GpioValue = 0x%08x\n", __FUNCTION__, mvGppValueGet(0, 0xffff)));
	mvGppValueSet(0, 
		(BIT(BIT_LED_ALARM) | BIT(BIT_LED_INFO) | 
		 BIT(BIT_LED_PWR)  | BIT(BIT_HDD_PWR)  |
		 BIT(BIT_FAN_LCK)) , 
		((MV_GPP_OUT & BIT(BIT_LED_ALARM)) | (MV_GPP_OUT & BIT(BIT_LED_INFO)) | 
		 (MV_GPP_OUT & BIT(BIT_LED_PWR))   | (MV_GPP_OUT & BIT(BIT_HDD_PWR))  |
		 (MV_GPP_IN & BIT(BIT_FAN_LCK)) ));
	BUF_TRACE(printk("%s > Setted up GpioValue = 0x%08x\n", __FUNCTION__, mvGppValueGet(0, 0xffff)));
	//MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_FAN_LCK)));
}

void BuffaloGpio_AlarmLedEnable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_ALARM)));
	MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_ALARM)));
}

void BuffaloGpio_AlarmLedDisable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_ALARM)));
	MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_ALARM)));
}

void BuffaloGpio_AlarmLedBlinkEnable(void){
	MV_REG_BIT_SET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_ALARM)));
}

void BuffaloGpio_AlarmLedBlinkDisable(void){
	MV_REG_BIT_RESET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_ALARM)));
}

void BuffaloGpio_InfoLedEnable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_INFO)));
	MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_INFO)));
}

void BuffaloGpio_InfoLedDisable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_INFO)));
	MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_INFO)));
}

void BuffaloGpio_InfoLedBlinkEnable(void){
	MV_REG_BIT_SET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_INFO)));
}

void BuffaloGpio_InfoLedBlinkDisable(void){
	MV_REG_BIT_RESET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_INFO)));

}

void BuffaloGpio_PowerLedEnable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_PWR)));
	MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_PWR)));
}


void BuffaloGpio_PowerLedDisable(void){
	MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), (BIT(BIT_LED_PWR)));
	MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), (BIT(BIT_LED_PWR)));
}

void BuffaloGpio_PowerLedBlinkEnable(void){
	MV_REG_BIT_SET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_PWR)));
}

void BuffaloGpio_PowerLedBlinkDisable(void){
	MV_REG_BIT_RESET(GPP_BLINK_EN_REG(0), (BIT(BIT_LED_PWR)));
}

void BuffaloGpio_HddPowerOff(void){
	mvGppOutEnablle(0, BIT(BIT_HDD_PWR), (MV_GPP_OUT_DIS & BIT(BIT_HDD_PWR)));
	mvGppPolaritySet(0, BIT(BIT_HDD_PWR), (MV_GPP_IN_INVERT & BIT(BIT_HDD_PWR)));
}

void BuffaloGpio_HddPowerOn(void){
	mvGppOutEnablle(0, BIT(BIT_HDD_PWR), (MV_GPP_OUT_EN & BIT(BIT_HDD_PWR)));
	mvGppPolaritySet(0, BIT(BIT_HDD_PWR), (MV_GPP_IN_INVERT & BIT(BIT_HDD_PWR)));
}

#if defined CONFIG_BUFFALO_GPIO_DRIVER_DEBUG
void BuffaloGpio_PrintStatus(void){
	BUF_TRACE(printk("%s > GPP_DATA_OUT_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_REG(0))));
	BUF_TRACE(printk("%s > GPP_DATA_OUT_EN_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_EN_REG(0))));
	BUF_TRACE(printk("%s > GPP_DATA_IN_POL_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_DATA_IN_POL_REG(0))));
	BUF_TRACE(printk("%s > GPP_DATA_IN_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_DATA_IN_REG(0))));
	BUF_TRACE(printk("%s > GPP_INT_CAUSE_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_INT_CAUSE_REG(0))));
	BUF_TRACE(printk("%s > GPP_INT_MASK_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_INT_MASK_REG(0))));
	BUF_TRACE(printk("%s > GPP_INT_LVL_REG(0)=0x%08x\n", 
		__FUNCTION__, MV_REG_READ(GPP_INT_LVL_REG(0))));
}
#endif

void BuffaloGpio_CpuReset(void){
	BUF_TRACE(printk("%s > BIT_CPU_RESET=%d\n", __FUNCTION__, BIT_CPU_RESET));
	mvBoardReset();
}

void BuffaloGpio_CpuResetClear(void){
	mvGppOutEnablle(0, BIT(BIT_CPU_RESET), (MV_GPP_OUT_EN & BIT(BIT_CPU_RESET)));
	mvGppPolaritySet(0, BIT(BIT_CPU_RESET), (MV_GPP_IN_INVERT & BIT(BIT_CPU_RESET)));
}

void BuffaloGpio_EthLedOn(void){
	link_led_on_mvEthE1112Phy(0);
}

void BuffaloGpio_EthLedOff(void){
	link_led_off_mvEthE1112Phy(0);
}

void BuffaloGpio_AllLedOff(void){
	BuffaloGpio_AlarmLedDisable();
	printk("%s > AlarmLed offed\n", __FUNCTION__);
	BuffaloGpio_InfoLedDisable();
	printk("%s > InfoLed offed\n", __FUNCTION__);
	BuffaloGpio_PowerLedDisable();
	printk("%s > PowerLed offed\n", __FUNCTION__);
	BuffaloGpio_EthLedOff();
	printk("%s > EthLed offed\n", __FUNCTION__);
}

unsigned int BuffaloGpio_IsCpuHotBoot(void){
	unsigned int REG_VAL=MV_REG_READ(0x20104);
	printk("%s > REG_VAL=0x%08x\n", __FUNCTION__, REG_VAL);
	return ((REG_VAL) & (1 << 1));
}

uint8_t BuffaloGpio_ChangePowerStatus(uint8_t ChangeType){
	
	uint8_t MagicKey=0;

	switch(ChangeType){
	case POWER_STATUS_REBOOTING:
		MagicKey = MagicKeyReboot;
		break;
	case POWER_STATUS_REBOOTING_UBOOT_PASSED:
		MagicKey = MagicKeyRebootUbootPassed;
		break;
	case POWER_STATUS_NORMAL_STATE:
		MagicKey = MagicKeyNormalState;
		break;
	case POWER_STATUS_HW_POWER_OFF:
		MagicKey = MagicKeyHwPoff;
		break;
	case POWER_STATUS_SW_POWER_OFF:
		MagicKey = MagicKeySwPoff;
		break;
	case POWER_STATUS_SW_POFF_UBOOT_PASSED:
		MagicKey = MagicKeySWPoffUbootPassed;
		break;
	case POWER_STATUS_FWUPDATING:
		MagicKey = MagicKeyFWUpdating;
		break;
	case POWER_STATUS_REBOOT_REACHED_HALT:
		MagicKey = MagicKeyRebootReachedHalt;
		break;
	case POWER_STATUS_SW_POFF_REACHED_HALT:
		MagicKey = MagicKeySWPoffReachedHalt;
		break;
	}

	if(MagicKey){
		BufRtcDS1339AlarmBSet(MagicKey);
	}else{
		BufRtcDS1339AlarmBGet(&MagicKey);
	}

	BUF_TRACE(printk("%s > MagicKey = 0x%08x\n", __FUNCTION__, MagicKey));
	return MagicKey;
}

void BuffaloUartClose(unsigned int port){
	BuffaloUartCloseInternal(port);
}

EXPORT_SYMBOL(BuffaloGpio_HddPowerOff);
EXPORT_SYMBOL(BuffaloGpio_CpuReset);
EXPORT_SYMBOL(BuffaloGpio_AllLedOff);
