#ifndef __mvUsbDebug_h__
#define __mvUsbDebug_h__
/**HEADER*********************************************************
-----------------------------------------------------------------------------
 CONFIDENTIAL AND PROPRIETARY INFORMATION OF TRANSDIMENSION, INC. AND/OR ITS
 WHOLLY OWNED SUBSIDIARY SOFTCONNEX TECHNOLOGIES, INC. THIS NOTICE IS NOT TO
 BE DELETED, MODIFIED, MOVED OR CHANGED IN ANY WAY.

 (TransDimension, Inc. and SoftConnex Technologies, Inc. are referred to
 collectively below as TransDimension, Inc.) 
 Copyright (c) 1999 - 2004 by TransDimension, Inc. 

 This software is protected by copyright laws and international copyright 
 treaties, as well as other intellectual property laws and treaties.  This
 software is a CONFIDENTIAL, unpublished work of authorship, and with portions 
 constituting TRADE SECRETS of TransDimension, Inc..  Any unauthorized use, 
 disclosure, and/or reproduction of this software, or any part of this software;
 or distribution of this software in any form or by any means; or storage of 
 this software in any database or retrieval system, without the express written
 consent of, and license from, TransDimension, Inc. is strictly prohibited.  
 This software is protected under the copyright and/or trade secret laws in 
 other countries in addition to USA.  All Rights Reserved.  Failure to abide by
 the use, disclosure and/or reproduction restrictions may result in civil and
 or criminal penalties, and will be prosecuted to the maximum extent of the law.
 -------------------------------------------------------------------------------

*** Comments:      
***   This file contains definitions for debugging the software stack
***                                                               
**************************************************************************
**END*********************************************************/

#include "mvUsbTypes.h"

#define MV_USB_RT_DEBUG

/************************************************************
The following array is used to make a run time trace route
inside the USB stack.
*************************************************************/

#define ARC_DEBUG_FLAG_ANY      0x00000000

#define ARC_DEBUG_FLAG_TRACE    0x00000001
#define ARC_DEBUG_FLAG_CTRL     0x00000002
#define ARC_DEBUG_FLAG_RX       0x00000004
#define ARC_DEBUG_FLAG_TX       0x00000008
#define ARC_DEBUG_FLAG_STALL    0x00000010
#define ARC_DEBUG_FLAG_STATUS   0x00000020
#define ARC_DEBUG_FLAG_TRANSFER 0x00000040
#define ARC_DEBUG_FLAG_INIT     0x00000080
#define ARC_DEBUG_FLAG_ISR      0x00000100
#define ARC_DEBUG_FLAG_ERROR    0x00000200
#define ARC_DEBUG_FLAG_ADDR     0x00000400
#define ARC_DEBUG_FLAG_DUMP     0x00000800
#define ARC_DEBUG_FLAG_SETUP    0x00001000
#define ARC_DEBUG_FLAG_CLASS    0x00002000
#define ARC_DEBUG_FLAG_SPEED    0x00004000
#define ARC_DEBUG_FLAG_RESET    0x00008000
#define ARC_DEBUG_FLAG_SUSPEND  0x00010000
#define ARC_DEBUG_FLAG_RESUME   0x00020000
#define ARC_DEBUG_FLAG_EP0      0x00040000
#define ARC_DEBUG_FLAG_EP1      0x00080000
#define ARC_DEBUG_FLAG_STATS    0x00100000


#define ARC_DEBUG_FLAG_ALL      0xffffffff

extern uint_32  usbDebugFlags;

#ifdef MV_USB_RT_DEBUG
#   define ARC_DEBUG_CODE(flags, code)    	        \
        if( (usbDebugFlags & (flags)) == (flags) )  \
            code
#else
#   define ARC_DEBUG_CODE(flags, code)
#endif

#if defined(MV_USB_TRACE_LOG)

#define TRACE_ARRAY_SIZE 400
#define MAX_STRING_SIZE  132

extern uint_16 DEBUG_TRACE_ARRAY_COUNTER;
extern char    DEBUG_TRACE_ARRAY[TRACE_ARRAY_SIZE][MAX_STRING_SIZE];

#define ARC_DEBUG_TRACE(flags, format, x...)                                        \
{                                                                                   \
    if( (usbDebugFlags & (flags)) == (flags))                                   \
    {                                                                               \
        USB_sprintf(DEBUG_TRACE_ARRAY[DEBUG_TRACE_ARRAY_COUNTER], format, ##x);     \
        DEBUG_TRACE_ARRAY_COUNTER++;                                                \
        if(DEBUG_TRACE_ARRAY_COUNTER >= TRACE_ARRAY_SIZE)                           \
            {DEBUG_TRACE_ARRAY_COUNTER = 0;}                                        \
    }                                                                               \
}
                                                    
#elif defined(MV_USB_TRACE_PRINT)

#   define ARC_DEBUG_TRACE(flags, format, x...)           \
        if((usbDebugFlags & (flags)) == (flags))      \
            USB_printf(format, ##x)

/*if trace switch is not enabled define debug log trace to empty*/
#else
#   define ARC_DEBUG_TRACE(flags, fromat, x...)
#endif


/************************************************************
The following are global data structures that can be used
to copy data from stack on run time. This structure can
be analyzed at run time to see the state of various other
data structures in the memory.
*************************************************************/

#endif /* __mvUsbDebug_h__ */
/* EOF */
