/*
 *  Driver routines for BUFFALO Platform
 *
 *  Copyright (C)  BUFFALO INC.
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU General Public License (GPL), incorporated herein by reference.
 *  Drivers based on or derived from this code fall under the GPL and must
 *  retain the authorship, copyright and license notice.  This file is not
 *  a complete program and may only be used when the entire operating
 *  system is licensed under the GPL.
 *
 */


#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include "BoardDriver/BuffaloGpio.h"

/* Globals */
// same as Buffalo Kernel Ver.
#define BUFFALO_CORE_VER "0.01"
#define DRIVER_NAME	"Buffalo Led Controll Driver"
#define AUTHOR	"(C) BUFFALO INC."

/* Module parameters */
MODULE_AUTHOR(AUTHOR);
MODULE_DESCRIPTION(DRIVER_NAME);
MODULE_LICENSE("GPL");
MODULE_VERSION(BUFFALO_CORE_VER);

/* Definitions */
#define DEBUG

#define USE_PROC_BUFFALO

// ----------------------------------------------------

/* Definitions for DEBUG */
#ifdef DEBUG
 #define FUNCTRACE(x)  x

#else
 #define FUNCTRACE(x) 

#endif


/* Function prototypes */
extern struct proc_dir_entry *proc_buffalo;

#if defined CONFIG_BUFFALO_USE_GPIO_DRIVER && defined CONFIG_BUFFALO_USE_LED_DRIVER
static int BuffaloAlarmLedReadProc(char *page, char **start, off_t offset, int length)
{
	int len = 0;
	off_t begin = 0;
	unsigned int PinStat = buffalo_gpio_read();

	if(~PinStat & BIT(BIT_LED_ALARM))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");
	*start = page + (offset - begin);
	len -= (offset - begin);
	if (len > length)
		len = length;
	return (len);
}

static int BuffaloAlarmLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		printk("%s > Calling BuffaloGpio_AlarmLedEnable\n", __FUNCTION__);
		BuffaloGpio_AlarmLedEnable();
	}else if(strncmp(buffer, "off", 3) == 0){
		printk("%s > Calling BuffaloGpio_AlarmLedDisable\n", __FUNCTION__);
		BuffaloGpio_AlarmLedDisable();
	}
	return count;
}

static int BuffaloInfoLedReadProc(char *page, char **start, off_t offset, int length)
{
	int len = 0;
	off_t begin = 0;
	unsigned int PinStat = buffalo_gpio_read();

	if(~PinStat & BIT(BIT_LED_INFO))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");
	*start = page + (offset - begin);
	len -= (offset - begin);
	if (len > length)
		len = length;
	return (len);

}

static int BuffaloInfoLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		BuffaloGpio_InfoLedEnable();
	}else if(strncmp(buffer, "off", 3) == 0){
		BuffaloGpio_InfoLedDisable();
	}
	return count;
}

static int BuffaloPowerLedReadProc(char *page, char **start, off_t offset, int length)
{
	int len = 0;
	off_t begin = 0;
	unsigned int PinStat = buffalo_gpio_read();

	if(~PinStat & BIT(BIT_LED_PWR))
		len = sprintf(page, "on\n");
	else
		len = sprintf(page, "off\n");
	*start = page + (offset - begin);
	len -= (offset - begin);
	if (len > length)
		len = length;
	return (len);

}

static int BuffaloPowerLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		BuffaloGpio_PowerLedEnable();
	}else if(strncmp(buffer, "off", 3) == 0){
		BuffaloGpio_PowerLedDisable();
	}
	return count;
}


static int BuffaloAllLedReadProc(char *page, char **start, off_t offset, int length)
{
	int len = 0;
	off_t begin = 0;
	unsigned int PinStat = buffalo_gpio_read();

	if(~PinStat & BIT(BIT_LED_PWR))
		len += sprintf(page+len, "power led :on\n");
	else
		len += sprintf(page+len, "power led :off\n");

	if(~PinStat & BIT(BIT_LED_INFO))
		len += sprintf(page+len, "info  led :on\n");
	else
		len += sprintf(page+len, "info  led :off\n");
	
	if(~PinStat & BIT(BIT_LED_ALARM))
		len += sprintf(page+len, "alarm led :on\n");
	else
		len += sprintf(page+len, "alarm led :off\n");

	*start = page + (offset - begin);
	len -= (offset - begin);
	if (len > length)
		len = length;
	return (len);

}

static int BuffaloAllLedWriteProc(struct file *filep, const char *buffer, unsigned long count, void *data)
{
	if(strncmp(buffer, "on", 2) == 0 ){
		BuffaloGpio_AlarmLedEnable();
		BuffaloGpio_InfoLedEnable();
		BuffaloGpio_PowerLedEnable();
	}else if(strncmp(buffer, "off", 3) == 0){
		BuffaloGpio_AlarmLedDisable();
		BuffaloGpio_InfoLedDisable();
		BuffaloGpio_PowerLedDisable();
	}
	return count;
}

//----------------------------------------------------------------------
int __init buffaloLedDriver_init (void)
{	struct proc_dir_entry *led_proc, *led_all_proc, *power_led_proc, *alarm_led_proc, *info_led_proc, *hdd_ctrl_proc, *cpu_reset_proc;
	FUNCTRACE(printk(">%s\n",__FUNCTION__));
	
	if(proc_buffalo) // if it were not exist.
		proc_buffalo = proc_mkdir("buffalo", 0);
	if (!proc_buffalo) { // if it were fail.
		printk (KERN_ERR "cannot init /proc/buffalo\n");
		return -ENOMEM;
	}
	
	led_proc = proc_mkdir("buffalo/led", 0);	
	alarm_led_proc = create_proc_entry("buffalo/led/alarm", 0, 0);
	alarm_led_proc->read_proc = &BuffaloAlarmLedReadProc;
	alarm_led_proc->write_proc= &BuffaloAlarmLedWriteProc;

	info_led_proc = create_proc_entry("buffalo/led/info", 0, 0);
	info_led_proc->read_proc = &BuffaloInfoLedReadProc;
	info_led_proc->write_proc= &BuffaloInfoLedWriteProc;

	power_led_proc = create_proc_entry("buffalo/led/power", 0, 0);
	power_led_proc->read_proc = &BuffaloPowerLedReadProc;
	power_led_proc->write_proc= &BuffaloPowerLedWriteProc;

	led_all_proc = create_proc_entry("buffalo/led/all", 0, 0);
	led_all_proc->read_proc = &BuffaloAllLedReadProc;
	led_all_proc->write_proc= &BuffaloAllLedWriteProc;
	
	printk("%s %s Ver.%s installed.\n", DRIVER_NAME, AUTHOR, BUFFALO_CORE_VER);
	return 0;
}

void buffaloLedDriver_exit(void)
{
	FUNCTRACE(printk(">%s\n",__FUNCTION__));

	remove_proc_entry("buffalo/led/info", 0);
	remove_proc_entry("buffalo/led/alarm", 0);
	remove_proc_entry("buffalo/led", 0);
	
